@@ -289,13 +289,14 @@ def process_apigateway_invocation(func_arn, path, payload, stage, api_id, header
             'requestContext': request_context,
             'stageVariables': get_stage_variables(api_id, stage),
         }
+        LOG.debug('Running Lambda function %s from API Gateway invocation: %s %s' % (func_arn, method or 'GET', path))
         return run_lambda(event=event, context={}, func_arn=func_arn,
             asynchronous=not config.SYNCHRONOUS_API_GATEWAY_EVENTS)
     except Exception as e:
         LOG.warning('Unable to run Lambda function on API Gateway message: %s %s' % (e, traceback.format_exc()))
 
 
-def process_sns_notification(func_arn, topic_arn, subscription_arn, message,
+def process_sns_notification(func_arn, topic_arn, subscription_arn, message, message_id,
         message_attributes, unsubscribe_url, subject='',):
     event = {
         'Records': [{
@@ -304,7 +305,7 @@ def process_sns_notification(func_arn, topic_arn, subscription_arn, message,
             'EventSubscriptionArn': subscription_arn,
             'Sns': {
                 'Type': 'Notification',
-                'MessageId': str(uuid.uuid4()),
+                'MessageId': message_id,
                 'TopicArn': topic_arn,
                 'Subject': subject,
                 'Message': message,
@@ -340,6 +341,11 @@ def process_kinesis_records(records, stream_name):
                         {
                             'eventID': 'shardId-000000000000:{0}'.format(rec['sequenceNumber']),
                             'eventSourceARN': stream_arn,
+                            'eventSource': 'aws:kinesis',
+                            'eventVersion': '1.0',
+                            'eventName': 'aws:kinesis:record',
+                            'invokeIdentityArn': 'arn:aws:iam::{0}:role/lambda-role'.format(TEST_AWS_ACCOUNT_ID),
+                            'awsRegion': aws_stack.get_region(),
                             'kinesis': rec
                         }
                         for rec in chunk
@@ -620,7 +626,7 @@ def get_handler_function_from_name(handler_name, runtime=LAMBDA_DEFAULT_RUNTIME)
 
 def error_response(msg, code=500, error_type='InternalFailure'):
     LOG.warning(msg)
-    return aws_responses.flask_error_response(msg, code=code, error_type=error_type)
+    return aws_responses.flask_error_response_json(msg, code=code, error_type=error_type)
 
 
 def get_zip_bytes(function_code):
@@ -1490,11 +1496,12 @@ def untag_resource(version, arn):
     return jsonify({})
 
 
-@app.route('/2019-09-25/functions/<function>/event-invoke-config', methods=['PUT'])
+@app.route('/2019-09-25/functions/<function>/event-invoke-config', methods=['PUT', 'POST'])
 def put_function_event_invoke_config(function):
-    """ Updates the configuration for asynchronous invocation for a function
+    # TODO: resouce validation required to check if resource exists
+    """ Add/Updates the configuration for asynchronous invocation for a function
         ---
-        operationId: PutFunctionEventInvokeConfig
+        operationId: PutFunctionEventInvokeConfig | UpdateFunctionEventInvokeConfig
         parameters:
             - name: 'function'
               in: path
@@ -1506,6 +1513,9 @@ def put_function_event_invoke_config(function):
     data = json.loads(to_str(request.data))
     function_arn = func_arn(function)
     lambda_obj = ARN_TO_LAMBDA[function_arn]
+
+    if request.method == 'PUT':
+        response = lambda_obj.clear_function_event_invoke_config()
     response = lambda_obj.put_function_event_invoke_config(data)
 
     return jsonify({
@@ -1547,6 +1557,18 @@ def get_function_event_invoke_config(function):
     return jsonify(response)
 
 
+@app.route('/2019-09-25/functions/<function>/event-invoke-config', methods=['DELETE'])
+def delete_function_event_invoke_config(function):
+    try:
+        function_arn = func_arn(function)
+        lambda_obj = ARN_TO_LAMBDA[function_arn]
+    except Exception as e:
+        return error_response(str(e), 400)
+
+    lambda_obj.clear_function_event_invoke_config()
+    return Response('', status=204)
+
+
 def serve(port, quiet=True):
     from localstack.services import generic_proxy  # moved here to fix circular import errors
 
