@@ -3,16 +3,21 @@ import json
 import random
 import logging
 import threading
+import time
 from binascii import crc32
 from requests.models import Request, Response
 from localstack import config
 from localstack.utils.aws import aws_stack, aws_responses
-from localstack.utils.common import to_bytes, to_str, clone
+from localstack.utils.common import to_bytes, to_str, clone, select_attributes
 from localstack.utils.analytics import event_publisher
+from localstack.utils.bootstrap import is_api_enabled
 from localstack.services.awslambda import lambda_api
 from localstack.services.generic_proxy import ProxyListener
 from localstack.services.dynamodbstreams import dynamodbstreams_api
 
+# set up logger
+LOGGER = logging.getLogger(__name__)
+
 # cache table definitions - used for testing
 TABLE_DEFINITIONS = {}
 
@@ -22,8 +27,8 @@ TABLE_TAGS = {}
 # action header prefix
 ACTION_PREFIX = 'DynamoDB_20120810'
 
-# set up logger
-LOGGER = logging.getLogger(__name__)
+# maps global table names to configurations
+GLOBAL_TABLES = {}
 
 # list of actions subject to throughput limitations
 THROTTLED_ACTIONS = [
@@ -38,6 +43,20 @@ class ProxyListenerDynamoDB(ProxyListener):
     def __init__(self):
         self._table_ttl_map = {}
 
+    @staticmethod
+    def table_exists(ddb_client, table_name):
+        paginator = ddb_client.get_paginator('list_tables')
+        pages = paginator.paginate(
+            PaginationConfig={
+                'PageSize': 100
+            }
+        )
+        for page in pages:
+            table_names = page['TableNames']
+            if to_str(table_name) in table_names:
+                return True
+        return False
+
     def forward_request(self, method, path, data, headers):
         if path.startswith('/shell') or method == 'GET':
             if path == '/shell':
@@ -63,9 +82,21 @@ class ProxyListenerDynamoDB(ProxyListener):
 
         if action == '%s.CreateTable' % ACTION_PREFIX:
             # Check if table exists, to avoid error log output from DynamoDBLocal
-            table_names = ddb_client.list_tables()['TableNames']
-            if to_str(data['TableName']) in table_names:
-                return 200
+            if self.table_exists(ddb_client, data['TableName']):
+                return error_response(message='Table already created',
+                                      error_type='ResourceInUseException', code=400)
+
+        if action == '%s.CreateGlobalTable' % ACTION_PREFIX:
+            return create_global_table(data)
+
+        elif action == '%s.DescribeGlobalTable' % ACTION_PREFIX:
+            return describe_global_table(data)
+
+        elif action == '%s.ListGlobalTables' % ACTION_PREFIX:
+            return list_global_tables(data)
+
+        elif action == '%s.UpdateGlobalTable' % ACTION_PREFIX:
+            return update_global_table(data)
 
         elif action in ('%s.PutItem' % ACTION_PREFIX, '%s.UpdateItem' % ACTION_PREFIX, '%s.DeleteItem' % ACTION_PREFIX):
             # find an existing item and store it in a thread-local, so we can access it in return_response,
@@ -86,14 +117,12 @@ class ProxyListenerDynamoDB(ProxyListener):
 
         elif action == '%s.DescribeTable' % ACTION_PREFIX:
             # Check if table exists, to avoid error log output from DynamoDBLocal
-            table_names = ddb_client.list_tables()['TableNames']
-            if to_str(data['TableName']) not in table_names:
+            if not self.table_exists(ddb_client, data['TableName']):
                 return get_table_not_found_error()
 
         elif action == '%s.DeleteTable' % ACTION_PREFIX:
             # Check if table exists, to avoid error log output from DynamoDBLocal
-            table_names = ddb_client.list_tables()['TableNames']
-            if to_str(data['TableName']) not in table_names:
+            if not self.table_exists(ddb_client, data['TableName']):
                 return get_table_not_found_error()
 
         elif action == '%s.BatchWriteItem' % ACTION_PREFIX:
@@ -106,6 +135,14 @@ class ProxyListenerDynamoDB(ProxyListener):
                             existing_items.append(find_existing_item(inner_request, table_name))
             ProxyListenerDynamoDB.thread_local.existing_items = existing_items
 
+        elif action == '%s.Query' % ACTION_PREFIX:
+            if data.get('IndexName'):
+                if not is_index_query_valid(to_str(data['TableName']), data.get('Select')):
+                    return error_response(message='One or more parameter values were invalid: Select type '
+                                                  'ALL_ATTRIBUTES is not supported for global secondary index id-index '
+                                                  'because its projection type is not ALL',
+                                          error_type='ValidationException', code=400)
+
         elif action == '%s.TransactWriteItems' % ACTION_PREFIX:
             existing_items = []
             for item in data['TransactItems']:
@@ -193,10 +230,12 @@ class ProxyListenerDynamoDB(ProxyListener):
         if not action:
             return
 
+        # upgrade event version to 1.1
         record = {
             'eventID': '1',
-            'eventVersion': '1.0',
+            'eventVersion': '1.1',
             'dynamodb': {
+                'ApproximateCreationDateTime': time.time(),
                 'StreamViewType': 'NEW_AND_OLD_IMAGES',
                 'SizeBytes': -1
             },
@@ -207,12 +246,15 @@ class ProxyListenerDynamoDB(ProxyListener):
 
         if action == '%s.UpdateItem' % ACTION_PREFIX:
             if response.status_code == 200:
+                existing_item = self._thread_local('existing_item')
+                record['eventName'] = 'INSERT' if not existing_item else 'MODIFY'
+
                 updated_item = find_existing_item(data)
                 if not updated_item:
                     return
-                record['eventName'] = 'MODIFY'
                 record['dynamodb']['Keys'] = data['Key']
-                record['dynamodb']['OldImage'] = self._thread_local('existing_item')
+                if existing_item:
+                    record['dynamodb']['OldImage'] = existing_item
                 record['dynamodb']['NewImage'] = updated_item
                 record['dynamodb']['SizeBytes'] = len(json.dumps(updated_item))
         elif action == '%s.BatchWriteItem' % ACTION_PREFIX:
@@ -270,7 +312,7 @@ class ProxyListenerDynamoDB(ProxyListener):
             event_publisher.fire_event(event_publisher.EVENT_DYNAMODB_CREATE_TABLE,
                 payload={'n': event_publisher.get_hash(data['TableName'])})
 
-            if data.get('Tags'):
+            if data.get('Tags') and response.status_code == 200:
                 table_arn = json.loads(response._content)['TableDescription']['TableArn']
                 TABLE_TAGS[table_arn] = {tag['Key']: tag['Value'] for tag in data['Tags']}
 
@@ -321,6 +363,10 @@ class ProxyListenerDynamoDB(ProxyListener):
             forward_to_lambda(records)
             forward_to_ddb_stream(records)
 
+    # -------------
+    # UTIL METHODS
+    # -------------
+
     def prepare_batch_write_item_records(self, record, data):
         records = []
         i = 0
@@ -356,7 +402,10 @@ class ProxyListenerDynamoDB(ProxyListener):
 
     def prepare_transact_write_item_records(self, record, data):
         records = []
-        for i, request in enumerate(data['TransactItems']):
+        # Fix issue #2745: existing_items only contain the Put/Update/Delete records,
+        # so we will increase the index based on these events
+        i = 0
+        for request in data['TransactItems']:
             put_request = request.get('Put')
             if put_request:
                 existing_item = self._thread_local('existing_items')[i]
@@ -372,6 +421,7 @@ class ProxyListenerDynamoDB(ProxyListener):
                     new_record['dynamodb']['OldImage'] = existing_item
                 new_record['eventSourceARN'] = aws_stack.dynamodb_table_arn(table_name)
                 records.append(new_record)
+                i += 1
             update_request = request.get('Update')
             if update_request:
                 table_name = update_request['TableName']
@@ -388,6 +438,7 @@ class ProxyListenerDynamoDB(ProxyListener):
                 new_record['dynamodb']['NewImage'] = updated_item
                 new_record['eventSourceARN'] = aws_stack.dynamodb_table_arn(table_name)
                 records.append(new_record)
+                i += 1
             delete_request = request.get('Delete')
             if delete_request:
                 table_name = delete_request['TableName']
@@ -400,10 +451,15 @@ class ProxyListenerDynamoDB(ProxyListener):
                 new_record['dynamodb']['OldImage'] = self._thread_local('existing_items')[i]
                 new_record['eventSourceARN'] = aws_stack.dynamodb_table_arn(table_name)
                 records.append(new_record)
+                i += 1
         return records
 
     def delete_all_event_source_mappings(self, table_arn):
         if table_arn:
+            # fix start dynamodb service without lambda
+            if not is_api_enabled('lambda'):
+                return
+
             lambda_client = aws_stack.connect_to_service('lambda')
             result = lambda_client.list_event_source_mappings(EventSourceArn=table_arn)
             for event in result['EventSourceMappings']:
@@ -418,6 +474,70 @@ class ProxyListenerDynamoDB(ProxyListener):
             return default
 
 
+def create_global_table(data):
+    table_name = data['GlobalTableName']
+    if table_name in GLOBAL_TABLES:
+        return get_error_message('Global Table with this name already exists', 'GlobalTableAlreadyExistsException')
+    GLOBAL_TABLES[table_name] = data
+    for group in data.get('ReplicationGroup', []):
+        group['ReplicaStatus'] = 'ACTIVE'
+        group['ReplicaStatusDescription'] = 'Replica active'
+    result = {'GlobalTableDescription': data}
+    return result
+
+
+def describe_global_table(data):
+    table_name = data['GlobalTableName']
+    details = GLOBAL_TABLES.get(table_name)
+    if not details:
+        return get_error_message('Global Table with this name does not exist', 'GlobalTableNotFoundException')
+    result = {'GlobalTableDescription': details}
+    return result
+
+
+def list_global_tables(data):
+    result = [select_attributes(tab, ['GlobalTableName', 'ReplicationGroup']) for tab in GLOBAL_TABLES.values()]
+    result = {'GlobalTables': result}
+    return result
+
+
+def update_global_table(data):
+    table_name = data['GlobalTableName']
+    details = GLOBAL_TABLES.get(table_name)
+    if not details:
+        return get_error_message('Global Table with this name does not exist', 'GlobalTableNotFoundException')
+    for update in data.get('ReplicaUpdates', []):
+        repl_group = details['ReplicationGroup']
+        # delete existing
+        delete = update.get('Delete')
+        if delete:
+            details['ReplicationGroup'] = [g for g in repl_group if g['RegionName'] != delete['RegionName']]
+        # create new
+        create = update.get('Create')
+        if create:
+            exists = [g for g in repl_group if g['RegionName'] == create['RegionName']]
+            if exists:
+                continue
+            new_group = {
+                'RegionName': create['RegionName'], 'ReplicaStatus': 'ACTIVE',
+                'ReplicaStatusDescription': 'Replica active'
+            }
+            details['ReplicationGroup'].append(new_group)
+    result = {'GlobalTableDescription': details}
+    return result
+
+
+def is_index_query_valid(table_name, index_query_type):
+    ddb_client = aws_stack.connect_to_service('dynamodb')
+
+    schema = ddb_client.describe_table(TableName=table_name)
+    for index in schema['Table'].get('GlobalSecondaryIndexes', []):
+        index_projection_type = index.get('Projection').get('ProjectionType')
+        if index_query_type == 'ALL_ATTRIBUTES' and index_projection_type != 'ALL':
+            return False
+    return True
+
+
 def find_existing_item(put_item, table_name=None):
     table_name = table_name or put_item['TableName']
     ddb_client = aws_stack.connect_to_service('dynamodb')
@@ -445,20 +565,24 @@ def find_existing_item(put_item, table_name=None):
     if 'Item' not in existing_item:
         if 'message' in existing_item:
             table_names = ddb_client.list_tables()['TableNames']
-            msg = ('Unable to get item from DynamoDB (existing tables: %s): %s' %
+            msg = ('Unable to get item from DynamoDB (existing tables: %s ...truncated if >100 tables): %s' %
                 (table_names, existing_item['message']))
             LOGGER.warning(msg)
         return
     return existing_item.get('Item')
 
 
-def get_table_not_found_error():
-    response = error_response(message='Cannot do operations on a non-existent table',
-                              error_type='ResourceNotFoundException')
+def get_error_message(message, error_type):
+    response = error_response(message=message, error_type=error_type)
     fix_headers_for_updated_response(response)
     return response
 
 
+def get_table_not_found_error():
+    return get_error_message(message='Cannot do operations on a non-existent table',
+                             error_type='ResourceNotFoundException')
+
+
 def fix_headers_for_updated_response(response):
     response.headers['content-length'] = len(to_bytes(response.content))
     response.headers['x-amz-crc32'] = calculate_crc32(response)
